/****************************************************************************** 
 *                      Sources BCB - www.developpez.com                      * 
 * -------------------------------------------------------------------------- * 
 *                                                                            *
 * Fractal.cpp                                                                *
 * Version : 1.0.0 mise  jour le 11 avril 2005                               *
 * Copyright Cryptonyx 2005 (http://five.chez.tiscali.fr)                     *
 *                                                                            * 
 * Test avec:                                                                * 
 * SE: WinXP                                                                  *
 * IDE: BCB6                                                                  *
 *                                                                            * 
 * -------------------------------------------------------------------------- * 
 *                                                                            *
 * Algorithmes de cration de fractales                                       *
 *                                                                            *
 * -------------------------------------------------------------------------- * 
 *                                                                            * 
 * Retrouvez d'autres ressources BCB sur :                                    * 
 *  - http://c.developpez.com/sources/bcb/                                    * 
 *  - http://c.developpez.com/faq/bcb/                                        * 
 * Ainsi que sur les forums (http://www.developpez.net/forums/)               * 
 *                                                                            * 
 ******************************************************************************/
//---------------------------------------------------------------------------
#include <math.h>
#include "fractal.h"
//---------------------------------------------------------------------------
TFractal *Fractal;
//---------------------------------------------------------------------------
__fastcall TFractal::TFractal()
{
}

//---------------------------------------------------------------------------
void __fastcall TFractal::InitDrawArea(TCanvas *Canvas, TColor Color, int Width, int Height)
{
   FractalZone = new TCanvas;
   FractalZone = Canvas;
   MaxX = Width;
   MaxY = Height;
   BaseColor = Color;
   if(BaseColor == clBlack)
      BaseColor = clYellow;
}

//---------------------------------------------------------------------------
void __fastcall TFractal::Formula(FractalTest SelectedTest)
{
   Test = SelectedTest;
}

//---------------------------------------------------------------------------
void __fastcall TFractal::GenerateJuliaFractal()
{
register float x, y;

   FractalZone->Pen->Color = clBlack;
   FractalZone->FillRect(Rect(0, 0, MaxX, MaxY));
   if(Zoom == 0)
      Zoom = 500;

   for(y = MaxY/2 + YDecay; y >= -MaxY/2 + YDecay; y--) {
      for(x = -MaxX/2 + XDecay; x <= MaxX/2 + XDecay; x++) {
         realZ = x / Zoom;
         imagZ = y / Zoom;
         if(ColorSelectionTest() == false)
            FractalZone->Pen->Color = clBlack;
         DrawPoint(x + MaxX / 2 - XDecay, y + MaxY / 2 - YDecay);
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFractal::GenerateMandelbrotFractal()
{
register int x, y;

   FractalZone->Pen->Color = clBlack;
   FractalZone->FillRect(Rect(0, 0, MaxX, MaxY));
   if(Zoom == 0)
      Zoom = 500;

   for(y = MaxY/2 + YDecay; y >= -MaxY/2 + YDecay; y--) {
      for(x = -MaxX/2 + XDecay; x <= MaxX/2 + XDecay; x++) {
         realZ = 0;
         imagZ = 0;
         realC = x / Zoom;
         imagC = y / Zoom;
         if(ColorSelectionTest() == false)
            FractalZone->Pen->Color = clBlack;
         DrawPoint(x + MaxX / 2 - XDecay, y + MaxY / 2 - YDecay);
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFractal::DrawPoint(int WhereX, int WhereY)
{
   FractalZone->MoveTo(WhereX, WhereY);
   FractalZone->Pixels[WhereX][WhereY] = FractalZone->Pen->Color;
}

//---------------------------------------------------------------------------
bool __fastcall TFractal::ColorSelectionTest()
{
float TempX, TempY, X2, Y2, Sum;
static int Index;
static TColor Color = BaseColor;

   for(Index = 1; Index <= 500; Index++) {
      // Z(n + 1) = Z2(n) + C
      // (a + ib)2 = a2 - b2 + 2i(ab)
      TempX = (realZ * realZ) - (imagZ * imagZ) + realC;
      TempY = 2 * realZ * imagZ + imagC;
      X2 = TempX * TempX;
      Y2 = TempY * TempY;
      realZ = TempX;
      imagZ = TempY;
      switch(Test) {
         case fFirstRemainderTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = BaseColor + (TColor)((int)(fmod(TempX + TempY, Index)));   // extraordinaire
         break;
         case fSecondRemainderTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)(BaseColor + (int)(fmod(TempX + TempY, BaseColor + Index)));   // extraordinaire
         break;
         case fThirdRemainderTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)((int)(fmod(X2 + Y2, (Index * Index)) + BaseColor));
         break;
         case fSquareRootTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)(Index * sqrt(Index) + BaseColor);
         break;
         case fFlatTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)(Index + BaseColor);
         break;
         case fSquareTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)(Index * Index + BaseColor);
         break;
         case fCubeTest:
            Sum = X2 * X2 + Y2 * Y2;
            Color = (TColor)(Index * Index * Index + BaseColor);
         break;
         case fFirstWarp:
            Sum = X2 * X2 + Y2;
            Color = (TColor)((int)(fmod(TempX + TempY, BaseColor + Index)));
         break;
         case fSecondWarp:
            Sum = X2 * X2 + Y2;
            Color = (TColor)(BaseColor + (int)(fmod(TempX + TempY, BaseColor + Index)));
         break;
         default:
            Sum = X2 + Y2;
            Color = (TColor)(BaseColor + (int)(fmod(TempX + TempY, Index)));
         break;
      }
      if(Sum > SetSize){
         FractalZone->Pen->Color = Color;
         return true;
      }
   }
   FractalZone->Pen->Color = clBlack;
   return false;
}

//---------------------------------------------------------------------------
// GenerateLyapounovFractal
//   avertissement : ce calcul dpend des valeurs IterationX et IterationY
//                   il est gnralement trs long (plusieurs minutes au moins)
//---------------------------------------------------------------------------
void __fastcall TFractal::GenerateLyapounovFractal()
{
double StretchX;
double StretchY;

double x;
double Exponent, Total, Root, Lyap;
double ConstX, ConstY;
int i, j, Iteration;
int Pattern, Rounds = 10; //  faire varier pour modifier les couleurs

   if(XInitValue <= 0 || XInitValue >= 1)
      XInitValue = 0.5;
   if(XLat <= 1 || XLat > 1.2)
      XLat = 1.02;
   if(YLat <= 1 || YLat > 1.2)
      YLat = 1.02;
   if(XBase < 1 || XBase > 4)
      XBase = 3.54;
   if(YBase <= 1 || YBase > 42)
      YBase = 3.45;

   ConstX = (XLat - 1) * XBase;
   ConstY = (YLat - 1) * YBase;

   for(j = 0; j < MaxY; j++) {
      // on fait varier le taux de croissance de YBase  YLat * YBase
      StretchY = (double(j) / double(MaxY)) * ConstY + YBase;

      for(i = 0; i < MaxX; i++) {
         // on fait varier le taux de croissance de XBase  XLat * XBase
         StretchX = (double(i) / double(MaxX)) * ConstX + XBase;
         
         // formule logistique : x(n+1) = r * x(n) * [1 - x(n)]
         // on calcule un x(n) avec n le plus grand possible
         x = XInitValue;
         for(Iteration = 1; Iteration < IterationX + 1; Iteration++)
            x = MarkusRoot(Iteration % 5, StretchX, StretchY) * x * (1 - x);

         // on calcule maintenant l'exposant de Lyapounov en introduisant
         // des ruptures comme dfinies par Markus
         Total = 0;
         for(Iteration = 1; Iteration < IterationY + 1; Iteration++) {
            Root = MarkusRoot(Iteration % 5, StretchX, StretchY);
            x = Root * x * (1 - x);
            Lyap = Root * (1 - 2 * x);
            if(Lyap > 0)
               Total = Total + (log(Lyap) / log(2));
            if(Lyap < 0)
               Total = Total + (log(-Lyap) / log(2));
         }
         Exponent = Total / IterationY;
         // Choix de la couleur d'affichage du point
  	      if(Exponent > 0)
	         Pattern = (TColor)((int)log(Exponent) * Rounds + BaseColor);
	      else if(Exponent < 0)
	         Pattern = (TColor)((int)log(-Exponent) * Rounds + BaseColor);

         FractalZone->Pen->Color = (TColor)(Pattern);
         DrawPoint(i, j);
      }
   }
}

//---------------------------------------------------------------------------
double TFractal::MarkusRoot(int i, double Root1, double Root2)
{
double Markus;

   // on choisit une "srie" de Markus de type 11212
   switch(i) {
      case 0:
      case 1:
      case 3:
         Markus = Root2;
      break;

      case 2:
      case 4:
      default:
         Markus = Root1;
      break;

   }

   return Markus;

}

